/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://kenai.com/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.observation.simple;

import javax.annotation.Nonnull;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.io.Serializable;
import it.tidalwave.netbeans.util.AsLookupSupport;
import it.tidalwave.observation.Finder;
import it.tidalwave.observation.Location;
import it.tidalwave.observation.Observation;
import it.tidalwave.observation.ObservationItem;
import it.tidalwave.observation.Observer;
import it.tidalwave.observation.Source;
import lombok.Getter;

/***********************************************************************************************************************
 *
 * @version $Id$ * @author  Fabrizio Giudici
 *
 **********************************************************************************************************************/
public class SimpleObservation extends AsLookupSupport implements Observation, Serializable
  {
    private static final long serialVersionUID = 1345454537593487L;

    @Nonnull
    private final SimpleObservationSet observationSet;

    @Nonnull
    private final Date date;

    @Nonnull @Getter
    private final Location location;

    @Nonnull
    private final Set<Observer> observers;

    @Nonnull
    private final Set<ObservationItem> observationItems;

    @Nonnull @Getter
    private final Source source;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public SimpleObservation (final @Nonnull SimpleObservationSet observationSet,
                              final @Nonnull Date date,
                              final @Nonnull Location location,
                              final @Nonnull Set<Observer> observers,
                              final @Nonnull Set<ObservationItem> observationItems,
                              final @Nonnull Source source,
                              final @Nonnull List<Object> extras)
      {
        super(extras.toArray());
        this.observationSet = observationSet;
        this.date = date;
        this.location = location;
        this.observers = observers;
        this.observationItems = observationItems;
        this.source = source;
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Date getDate()
      {
        return new Date(date.getTime());
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Finder<ObservationItem> findObservationItems()
      {
        return new SimpleFinderSupport<ObservationItem>(observationItems);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Finder<Observer> findObservers()
      {
        return new SimpleFinderSupport<Observer>(observers);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public ChangeBuilder change()
      {
        return new SimpleObservationChangeBuilder(observationSet, this);
      }
  }
